! This example shows how to use the int16Array2Data interface.
! In the example a dataset is created (opened) containing
! a table with 2 columns of two 2-dimensional arrays.
!
! The second array has the same data type as the first; this
! is ensured by using the arrayDataType() function to determine
! the data type of the first array.
!
! The columns are then  initialised, on a row-by-row
! basis (i.e. accessing the column's data cell-by-cell),
! before the dataset is released (closed).
program example_cellarray2data

  use dal
  use errorhandling

  implicit none

  type(DataSetT) set
  type(TableT) tab
  type(ColumnT) col1, col2
  integer(kind=INT16), dimension(:,:), pointer :: c1, c2
  integer, dimension(2), parameter :: s = (/ 3,4 /)
  integer :: i,j,k,n

  ! create a set
  set = dataSet("test.dat",CREATE)
  tab = addTable(set, "table", 100, "table comment" )
  col1 = addColumn( tab, "column1", INTEGER16, "km", s, "column comment" )
  col2 = addColumn( tab, "column2", columnDataType( col1 ), "km", s, "column comment" )

  ! fill with unique numbers

  n = 0
  do k=0,numberOfRows(tab) - 1
    c1 => int16Array2Data(col1,k)
    c2 => int16Array2Data(col2,k)
    do j=0,3
      do i=0,2 
        c1(i,j) = n
        c2(i,j) = c1(i,j)
        n = n + 1
      end do
    end do
  end do

  call release(col1)
  call release(col2)
  call release(set)

end program example_cellarray2data
