! In this example add dataset is created (opened) containing
! two 3-dimensional arrays, and one table.
!
! It illustrates the use of the numberofelements interface.
!
! The second array has the same data type as the first; this
! is ensured by using the arrayDataType() function to determine
! the data type of the first array.
program example_numberofdimensions

  use dal
  use errorhandling

  implicit none

  type(DataSetT) set
  type(ArrayT) arr
  type(TableT) tab
  type(ColumnT) col
  integer :: i,j

  ! create a set
  set = dataSet("test.dat",CREATE)
  arr = addArray(set, "array1", INTEGER32, dimensions=(/3/) )
  arr = addArray(set, "array2", dataType( arr ), dimensions=(/3,4/) )
  arr = addArray(set, "array3", dataType( arr ), dimensions=(/3,4,5/) )
  tab = addTable(set,"table",10)
  col = addColumn(tab,"col1",INTEGER8) ! scalar
  col = addColumn(tab,"col2",dataType(col),dimensions=(/3/)) ! vector
  col = addColumn(tab,"col3",dataType(col),dimensions=(/3,4/)) ! 2-dimensions
  col = addColumn(tab,"col4",dataType(col),dimensions=(/3,4,5/)) ! 3-dimensions
  col = addColumn(tab,"col5",dataType(col),dimensions=(/3,4,5,6/)) ! 4-dimensions

  do i = 0, numberOfBlocks( set ) - 1
    ! For each block which is an array, display the
    ! name, number of dimensions and the number of elements.
    if( blockType( set, i ).eq.ARRAY_BLOCK ) then
      arr = array( set, i, READ )
      write(*,*) name( arr ), numberOfDimensions( arr ), numberOfElements( arr )
    else
      tab = table( set, i )
      do j = 0, numberOfColumns( tab ) - 1
        ! For each column, display the name,
	! number of dimensions and total number of elements.
        col = column( tab, j, READ )
	write(*,*) name( col ), numberOfDimensions( col ), numberOfRows( col ) * numberOfElements( col )
      end do
    end if
  end do

  call release(set)

end program example_numberofdimensions
