using namespace std;
/*************************************************************** 
* 
* DalBlock.cc               & XMM Science Analysis System (SAS) 
* 
* Class representing a Block.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: DalBlock.cc,v 1.2 2004/03/05 14:43:00 eojero Exp $
* 
*****************************************************************/ 


#include "DalBlock.h"
#include <errstr.h>
#include <Tracer.h>
#include <SimpleScanner.h>
#include <stringutil.h>


DalBlock::DalBlock(
		const string &	name,
		Type		type,
		const string &	label,
		DalDataSet *	parent )

	:
		DalAttributable( name, label, parent -> mode(), parent ),
		DalChild< DataSet >( parent ),
		DalChild< DalDataSet >( parent ),
		_type( type )
{
	PROC
	ENDPROC
}


DalBlock& DalBlock::operator=( const DalBlock& org)
{
	PROC

	(Attributable&)*this = org;

	RETURN *this;
}
 

DalBlock::Type
DalBlock::type() const
{
	PROC RETURN _type;
}


unsigned int
DalBlock::blockNumber() const
{
	PROC
	DataSet * dataSet = ((DalChild<DataSet>&)*this).parent();
	RETURN dataSet -> blockNumber( name() );
}


void
DalBlock::printOn( ostream & os ) const
{
	PROC

	// ******************************
	// Write the Block Name and Type.
	// ******************************

	const string type = ( _type == TableT ? "TABLE" : "ARRAY" );

	os << "BLOCK: Name->" << name() << " Label-> " << label() << " Type->" << type << endl;

	// **************************
	// Write the block atributes.
	// **************************
	DalAttributable::printOn( os );

	ENDPROC

}


void
DalBlock::rename( const string & newName )
{
        PROC

	DataSet * dataSet = ((DalChild<DataSet>&)*this).parent();

        if( !dataSet -> hasBlock( newName ))
        {
                DalLabelled::rename( newName );
        }
        else
        {
                errstr << "It is not possible to rename the block with name '" << name() << "' to '" << newName << "', because a block with name '" << newName << "' already exists in the data set with name '" << dataSet -> name() << "'" << Sas::error( "invalidTableName" );
        } // if
        ENDPROC

}


string
DalBlock::parseName( const string & compoundName ) const
{
	PROC

	SimpleScanner	sc( compoundName, ":%" );
	string		parsedName;

	try
	{
		parsedName = sc.match( SimpleToken::String ).asString();

		if( sc.nextIsCharacter( ':' ) )
		{
			DalDataSet * parentDataSet = ((DalChild<DalDataSet>&)*this).parent();
			parsedName = parentDataSet -> parseName( compoundName );
			SimpleScanner	sc( parsedName, ":%" );

			if( sc.nextIsCharacter( ':' ) )
			{
				sc.skip();

				ALTRETURNV( sc.remainder() );
			} // if

			parsedName = sc.match( SimpleToken::String ).asString();

			if( parsedName != name() )
			{
				errstr << "The compound name '" << compoundName << "' is invalid; the qualified name of this object is " << qualifiedName() << Sas::error( "invalidCompoundName" );
			} // if
			if( sc.nextIsCharacter( '%' ) )
			{
				ALTRETURNV( sc.remainder() );
			} // if
			sc.match( ':' );
			ALTRETURNV( sc.remainder() );
		}
		RETURN compoundName;
	}
	catch( SimpleScanner::Expected e )
	{
		errstr << "An error was detected with the name '" << compoundName << "'.  " << e.message() << Sas::error( "invalidCompoundName" );
	}

	RETURN "will not get here";

}


string
DalBlock::qualifiedName() const
{
	PROC
	RETURN DalChild<DataSet>::parent() -> qualifiedName() + ":" + name();
}



