#ifndef __DATABASE
#define __DATABASE

#include "Dal.h"
#include "Tracer.h"
#include "errstr.h"


class Database
{
	public :

		virtual ~Database() { }
		virtual void read( DataSet* dataSet ) = 0;
		virtual void write( DataSet* dataSet ) = 0;
		void lock() { PROC _locked = true; ENDPROC }
		void unlock() { PROC _locked = false; ENDPROC }
		bool locked() const { PROC RETURN _locked; }
		void clobber( bool setting ) { PROC _clobber = setting; ENDPROC }
		bool clobber() const { PROC RETURN _clobber; }
		virtual void addEntry( const DataSet & dataSet ) = 0;
		virtual void addEntry( const Block & block ) = 0;
		virtual void addEntry( const Table & table ) = 0;
		virtual void addEntry( const Array & array ) = 0;
		virtual void addEntry( const Column & column ) = 0;
		virtual void setVariableColumnSize( const CellData & cellData ) = 0;
		virtual void deleteEntry( const DataSet & dataSet ) = 0;
		virtual void deleteEntry( const Block & block ) = 0;
		virtual void deleteEntry( const Table & table ) = 0;
		virtual void deleteEntry( const Array & array ) = 0;
		virtual void deleteEntry( const Column & column ) = 0;
		virtual bool hasEntry( const DataSet & dataSet ) = 0;
		virtual bool hasEntry( const Block & block ) = 0;
		virtual bool hasEntry( const Table & table ) = 0;
		virtual bool hasEntry( const Array & array ) = 0;
		virtual bool hasEntry( const Column & column ) = 0;

		virtual void insertRows(
				const Table &	table,
				unsigned long	pos,
				unsigned long	count ) = 0;

		virtual void deleteRows(
				const Table &	table,
				unsigned long	from,
				unsigned long	count ) = 0;

		virtual void copyRows(
				const Column &	column,
				unsigned long	from,
				unsigned long	to,
				unsigned long	count ) = 0;

		virtual void dumpData(
				const Column & 	column,
				char ** 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				bool8 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				int8 * 		values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				uint16 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				int16 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				uint32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				int32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				real32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				real64 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				char ** 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				bool8 *		values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				int8 * 		values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				uint16 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				int16 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				uint32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				int32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				real32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				real64 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				char ** 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				bool8 *		values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				int8 * 		values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				uint16 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				int16 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				uint32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				int32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				real32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				real64 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				char ** 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				bool8 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				int8 * 		values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				uint16 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				int16 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				uint32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				int32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				real32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				real64 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Array & 	array, 
				char ** 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Array & 	array, 
				bool8 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Array & 	array, 
				int8 * 		values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Array & 	array,
				uint16 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Array & 	array,
				int16 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Array & 	array,
				uint32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Array & 	array,
				int32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Array & 	array,
				real32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void dumpData(
				const Array & 	array,
				real64 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Array & 	array,
				char ** 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Array & 	array,
				bool8 *		values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Array & 	array,
				int8 * 		values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Array & 	array,
				uint16 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Array & 	array,
				int16 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Array & 	array,
				uint32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Array & 	array,
				int32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Array & 	array,
				real32 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;
		virtual void loadData(
				const Array &	array,
				real64 * 	values,
				unsigned long 	from,
				unsigned long 	count ) = 0;


	protected :

		DataSet *	_dataSet;

	private :

		bool		_locked;
		bool		_clobber;
};


#endif

