using namespace std;
/*

   ExtDal.cc

   Extended DAL containing iterators.

   (c) 1998 European Space Agency - written by Marco Beijersbergen
*/

#include <Tracer.h>
#include <errstr.h>

#include "ExtDal.h"		// DAL
#include "DalChild.h"
#include "DalSeekable.h"
#include "Tracer.h"


class DalSubTable : public virtual SubTable, public DalChild<Table>, public DalChild<SubTableProcessor>, public DalSeekable<unsigned long>
{
	public :
		DalSubTable(
			Table *			table,
			SubTableProcessor *	subTableProcessor );
		~DalSubTable();

		ColumnData * data( const string & name ) const;
		ColumnData * data( unsigned int columnNumber ) const;
		CellData * cellData( const string & name, unsigned long row );
		CellData * cellData( unsigned int columnNumber, unsigned long row );
		void seek( unsigned long from, unsigned long count );
		void initialise();

	private :
		//vector< ColumnData *>	_columnData;
		//CellData *		_cellData;
};


DalSubTable::DalSubTable(
		Table *			table,
		SubTableProcessor *	subTableProcessor )
	:	DalChild<Table>( table ),
		DalChild<SubTableProcessor>( subTableProcessor ),
		DalSeekable<unsigned long>( 0, 0 )//,
		//_cellData( 0 )
{
	PROC
	initialise();
	ENDPROC
}


void
DalSubTable::initialise()
{
	PROC
	//while( _columnData.size() > 0 )
	//{
	//	delete _columnData[ 0 ];
	//	_columnData.erase( _columnData.begin() );
	//} // while

	Table * table = DalChild<Table>::parent();

	//for( unsigned int i = 0; i < table -> columns(); ++i )
	//{
	//	Column * column = table -> column( i );
	//	ColumnData * columnData = column -> data();
	//	_columnData.push_back( columnData );
	//} // for
	seek( 0, table -> rows() );
	ENDPROC
}


ColumnData *
DalSubTable::data( const string & name ) const
{
	PROC
	const Column* column = DalChild<Table>::parent() -> column( name );
	//const unsigned int columnNumber = DalChild<Table>::parent() -> columnNumber( name );
	RETURN column -> data( from(), count() );
	//RETURN _columnData[ columnNumber ];
}


ColumnData *
DalSubTable:: data( unsigned int columnNumber ) const
{
	PROC
	const Column* column = DalChild<Table>::parent() -> column( columnNumber );
	RETURN column -> data( from(), count() );
	//RETURN _columnData[ columnNumber ];
}


CellData *
DalSubTable::cellData( const string & name, unsigned long row )
{
	PROC
	Column* column = DalChild<Table>::parent() -> column( name );
	RETURN column -> cellData( row );
	//delete _cellData;
	//_cellData = DalChild<Table>::parent() -> column( name ) -> cellData( row );
	//RETURN _cellData;
}


CellData *
DalSubTable::cellData( unsigned int columnNumber, unsigned long row )
{
	PROC
	Column* column = DalChild<Table>::parent() -> column( columnNumber );
	RETURN column -> cellData( row );
	//delete _cellData;
	//_cellData = DalChild<Table>::parent() -> column( columnNumber ) -> cellData( row );
	//RETURN _cellData;
}


DalSubTable::~DalSubTable()
{
	PROC
	//for( unsigned int i = 0; i < _columnData.size(); ++i )
	//{
	//	delete _columnData[ i ];
	//} // for
	//delete _cellData;
	ENDPROC
}


void
DalSubTable::seek( unsigned long from, unsigned long count )
{
	PROC
	//for( unsigned int i = 0; i < _columnData.size(); ++i )
	//{
	//	_columnData[ i ] -> seek( from, count );
	//} // for
	DalSeekable<unsigned long>::seek( from, count );
	ENDPROC
}


SubTableProcessor::SubTableProcessor( Table * table )
	:	_subTable( new DalSubTable( table, this )),
		_from( 0 )
{
	PROC

	const char* model = getenv("SAS_MEMORY_MODEL");

        if( model ) _model.type( model );

	ENDPROC
}


bool
SubTableProcessor::next()
{
	PROC

	const char * dal_rows = getenv( "SAS_ROWS" );
	const unsigned long seekSize = (dal_rows ? atoi( dal_rows ) : 1 );
	const unsigned long rows = ((Child<Table>&)*_subTable).parent() -> rows();

	if( _from >= rows )
	{
		//_subTable -> seek( 0, rows );
		_subTable -> initialise();

		ALTRETURNV( false );
	} // if

	const unsigned long seekThisMany = (( _from + seekSize <= rows ) ? seekSize : rows - _from );
	_subTable -> seek( _from, seekThisMany );
	_from += seekSize;
	
	RETURN true;

}


SubTable *
SubTableProcessor::subTable() const
{
	PROC
	RETURN _subTable;
}


void
SubTableProcessor::forEachSubTable()
{
	PROC

	switch( _model.type() )
	{
		case Model::High :
		case Model::Unspecified :
			proc( _subTable );
			break;
		case Model::HighLow :
		case Model::Low :
		{
			const char * dal_rows = getenv( "SAS_ROWS" );
			const unsigned long seekSize = (dal_rows ? atoi( dal_rows ) : 1 );

			const unsigned long rows = ((Child<Table>&)*_subTable).parent() -> rows();

			for( unsigned long from = 0; from < rows; from = from + seekSize )
			{
				const unsigned long seekThisMany = (( from + seekSize <= rows) ? seekSize : rows - from );
				_subTable -> seek( from, seekThisMany );
				proc( _subTable );
			} // for
			_subTable -> initialise();
			//_subTable -> seek( 0, rows );
			break;
		}
		default :
			errstr << "Invalid Memory Model" << Sas::error( "InvalidMemoryModel" );
			break;
	} // switch
	ENDPROC
}


void
SubTableProcessor::forEachRow()
{
	PROC

	switch( _model.type() )
	{
		case Model::HighLow :
		case Model::High :
		case Model::Low :
		case Model::Unspecified :
		{
			const unsigned long rows = ((Child<Table>&)*_subTable).parent() -> rows();

			for( unsigned long from = 0; from < rows; ++from )
			{
				_subTable -> seek( from, 1 );
				proc( _subTable );
			} // for
			//_subTable -> seek( 0, rows );
			_subTable -> initialise();
			break;
		}
		default :
			errstr << "Invalid Memory Model" << Sas::error( "InvalidMemoryModel" );
			break;
	} // switch
	ENDPROC
}


// ------------- BLOCK ITERATION BEYOND HERE -------------


Table*
table( DataSet* dataSet, unsigned int blockNumber )
{
	PROC
	RETURN dataSet -> table( blockNumber );
}


Array*
array( DataSet* dataSet, unsigned int blockNumber )
{
	PROC
	RETURN dataSet -> array( blockNumber );
}


//template<class T, Block::Type S, T* (DataSet::*B)(unsigned int)>
template<class T, Block::Type S, T*(*B)(DataSet*,unsigned int)>
//template<class T, Block::Type S, class B>
class DalBlockIterator : public virtual BlockIterator<T>
{
	public :
		DalBlockIterator( DataSet* dataSet );
		T* next();
		void begin();
	private :
		DataSet*	_dataSet;
		T*		_current;
};


//template<class T, Block::Type S, T* (DataSet::*B)(unsigned int)>
template<class T, Block::Type S, T*(*B)(DataSet*,unsigned int)>
//template<class T, Block::Type S, class B>
DalBlockIterator<T,S,B>::DalBlockIterator( DataSet* dataSet )
	: _dataSet( dataSet ),
	  _current( 0 )
{
	PROC
	ENDPROC
}


//template<class T, Block::Type S, T* (DataSet::*B)(unsigned int)>
template<class T, Block::Type S, T*(*B)(DataSet*,unsigned int)>
//template<class T, Block::Type S, class B>
T*
DalBlockIterator<T,S,B>::next()
{
	PROC

	T* nextBlock( 0 );

	if( !_current )
	{
		begin();
		nextBlock = _current;
	}
	else
	{
		for(	unsigned int i = _current -> blockNumber() + 1;
			i < _dataSet -> blocks();
			++i )
		{
			Block* block( _dataSet -> block( i ) );
			if( block -> type() == S )
			{
				//_current = (_dataSet ->* B)( i );
				_current = B( _dataSet, i );
				nextBlock = _current;
				break;
			} // if
		} // for
	} // if
	RETURN nextBlock;
}


//template<class T, Block::Type S, T* (DataSet::*B)(unsigned int)>
template<class T, Block::Type S, T*(*B)(DataSet*,unsigned int)>
//template<class T, Block::Type S, class B>
void
DalBlockIterator<T,S,B>::begin()
{
	PROC
	for( unsigned int i = 0; i < _dataSet -> blocks(); ++i )
	{
		Block* block( _dataSet -> block( i ) );
		if( block -> type() == S )
		{
			//_current = (_dataSet ->* B)( i );
			_current = B( _dataSet, i );
			break;
		} // if
	} // for
	ENDPROC
}


//class DalTableIterator : public virtual TableIterator, public DalBlockIterator<Table,Block::TableT,&DataSet::table>
class DalTableIterator : public virtual TableIterator, public DalBlockIterator<Table,Block::TableT,table>
{
	//DalBlockIterator( DataSet* dataSet ) : DalBlockIterator<Table,Block::TableT,table>( dataSet ) {}
	public :
		DalTableIterator( DataSet* dataSet ) : DalBlockIterator<Table,Block::TableT,table>( dataSet ) {}
};


//class DalArrayIterator : public virtual ArrayIterator, public DalBlockIterator<Array,Block::ArrayT,&DataSet::array>
class DalArrayIterator : public virtual ArrayIterator, public DalBlockIterator<Array,Block::ArrayT,array>
{
	public :
		DalArrayIterator( DataSet* dataSet ) : DalBlockIterator<Array,Block::ArrayT,array>( dataSet ) {}
};


/*
class DalTableIterator : public virtual TableIterator
{
	public :
		DalTableIterator( DataSet* dataSet );
		Table* next();
		void begin();
	private :
		DataSet*	_dataSet;
		Table*		_current;
};


class DalArrayIterator : public virtual ArrayIterator
{
	public :
		DalArrayIterator( DataSet* dataSet );
		Array* next();
		void begin();
	private :
		DataSet*	_dataSet;
		Table*		_current;
};


DalTableIterator::DalTableIterator( DataSet* dataSet )
	: _dataSet( dataSet ),
	  _current( 0 )
{
	PROC
	ENDPROC
}


Table*
DalTableIterator::next()
{
	PROC

	Table* nextTable( 0 );

	if( !_current )
	{
		begin();
	} // if

	if( _current )
	{
		for(	unsigned int i = _current -> blockNumber() + 1;
			i < _dataSet -> blocks() - _current -> blockNumber();
			++i )
		{
			Block* block( _dataSet -> block( i ) );
			if( block -> type() == Block::TableT )
			{
				_current = _dataSet -> table( i );
				break;
			} // if
		} // for
	} // if
	RETURN nextTable;
}


void
DalTableIterator::begin()
{
	PROC
	for( unsigned int i = 0; i < _dataSet -> blocks(); ++i )
	{
		Block* block( _dataSet -> block( i ) );
		if( block -> type() == Block::TableT )
		{
			_current = _dataSet -> table( i );
			break;
		} // if
	} // for
	ENDPROC
}
*/


class DalBlockIteratorServer : public virtual BlockIteratorServer
{
	public :
		ArrayIterator * arrayIterator( DataSet* dataSet ) const
		{
			PROC
			RETURN new DalArrayIterator( dataSet );
		}
		TableIterator * tableIterator( DataSet* dataSet ) const
		{
			PROC
			RETURN new DalTableIterator( dataSet );
		}
};


BlockIteratorServer* blockIteratorServer = new DalBlockIteratorServer;

