using namespace std;
/*************************************************************** 
* 
* MemColumnData.cc           & XMM Science Analysis System (SAS) 
* 
* Class representing the Data in a Column. 
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: MemColumnData.cc,v 1.2 2004/03/05 14:43:03 eojero Exp $ 
* 
*****************************************************************/ 


#include <errstr.h>
#include "vectorutil.h"
#include "MemColumnData.h"

#include <cassert>

MemColumnData::MemColumnData(
			DataStorage *			data,
			const DalColumn *		parent,
			unsigned long			from,
			unsigned long			count,
			AccessMode 			accessMode )

		:	DalColumnData( parent, from, count ),
			MemData<DalColumn>( data, 0, 0, parent, accessMode )
{
	PROC

	seek( from, count);

	ENDPROC
}


MemColumnData::~MemColumnData()
{
	PROC

	DalColumn* column = this -> DalChild<DalColumn>::parent();
	column -> removeDataObject( this );

	ENDPROC
}


unsigned long
MemColumnData::position(
		unsigned long			row) const
{
	PROC

	if ( elements() == 0 )
		Sas::error("noVarSizeColumn","var-size cells not supported");

	assert( row >= from() && row < from() + count() );

	RETURN (row - from()) * elements();
}

unsigned long
MemColumnData::position(
		unsigned long			row,
		unsigned long			pos) const
{
	PROC

	if ( elements() == 0 )
		Sas::error("noVarSizeColumn","var-size cells not supported");

	assert( row >= from() && row < from() + count() && pos < elements() );

	RETURN ( row - from() ) * elements() + pos;

}


unsigned long
MemColumnData::position(
		unsigned long			row,
		const vector<unsigned long>& 	pos) const
{
	PROC

	const Column * column = this -> DalChild<Column>::parent();

	if ( elements() == 0 )
		Sas::error("noVarSizeColumn","var-size cells not supported");

	if ( row < from() || row >= from() + count() )
	{
 		errstr << "cannot access row " << row << " since the data ranges from " << from() <<" to " << from()+count() << Sas::error("invalidRow");
	} // if

	unsigned dim = size().size();

	if ( pos.size() != dim )
	{
		errstr << "the position has dimension " << pos.size() << " whereas the column " << column->name() << " has dimension " << dim << Sas::error("invalidPosition");
	} // if

	unsigned long factor = 1;

	unsigned long idx = 0;

	for (unsigned i = dim; i; --i)
	{
		unsigned ix = i-1;
		if ( pos[ix] >= size()[ix] )
		{
			errstr << "cannot access position" << pos << " since the cell data has size" << size() << Sas::error("invalidPosition");

		} // if

		idx += factor * pos[ix];
		factor *= size()[ix];
	} // for

	idx += (row - from()) * factor;

	RETURN idx;
}


void
MemColumnData::seek( unsigned long from, unsigned long count )
{
	PROC

	unsigned long	_from;
	unsigned long	_count;
	Column *	column = this -> DalChild<Column>::parent();


	if( count == 0 )
	{
		if( column -> count() == 0 )
		{
			_from = 0;
			_count = column -> rows();
		}
		else
		{
			_from = column -> from();
			_count = column -> count();
		} // if
	}
	else
	{
		_from = from;
		_count = count;
	} // if

	DalSeekable< unsigned long >::seek( _from, _count );

	seekable().seek(
		DalSeekable<unsigned long>::from() * column -> elements(),
		DalSeekable<unsigned long>::count() * column -> elements() );

	ENDPROC
}


unsigned long
MemColumnData::elements() const
{
	PROC
	RETURN this -> DalChild<Column>::parent() -> elements();
}


const vector<unsigned long>&
MemColumnData::size() const
{
	PROC
	RETURN this -> DalChild<Column>::parent() -> size();
}


