/*************************************************************** 
* 
* MemData.h                  & XMM Science Analysis System (SAS) 
* 
* Class representing data object.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: MemData.h,v 1.3 2009/06/22 15:58:03 rsaxton Exp $ 
* 
*****************************************************************/ 
#ifndef MEM_DATA_H
#define MEM_DATA_H


#include "Dal.h"
#include "DataStorage.h"
#include "Tracer.h"
#include "DalData.h"


/*AI gcc 4 porting 
... _data -> int8Data( seekable().from(), seekable().count() ); } => _data -> int8Data( DalData<T>::seekable().from(), DalData<T>::seekable().count() ); }
*/
template<class T>
class MemData : public DalData<T>
{
	public :

		// Constructors
		MemData(
			DataStorage *			data,
			unsigned long			from,
			unsigned long			count,
			const T *			parent,
			Dal::AccessMode			accessMode )

			: DalData<T>( parent, from, count, accessMode ),
			  _data( data )
		{
			PROC
			ENDPROC
		}
		virtual ~MemData(){}

		void reset( DataStorage *	data ) { PROC _data = data; ENDPROC }
		DataStorage * dataStorage() const { PROC RETURN _data; }

		int8 * int8Data() const { PROC RETURN _data -> int8Data( DalData<T>::seekable().from(), DalData<T>::seekable().count() ); }
		uint16 * uint16Data() const { PROC RETURN _data -> uint16Data( DalData<T>::seekable().from(), DalData<T>::seekable().count() ); }
		int16 * int16Data() const { PROC RETURN _data -> int16Data( DalData<T>::seekable().from(), DalData<T>::seekable().count() ); }
		uint32 * uint32Data() const { PROC RETURN _data -> uint32Data( DalData<T>::seekable().from(), DalData<T>::seekable().count() ); }
		int32 * int32Data() const { PROC RETURN _data -> int32Data( DalData<T>::seekable().from(), DalData<T>::seekable().count() ); }
		real32 * real32Data() const { PROC RETURN _data -> real32Data( DalData<T>::seekable().from(), DalData<T>::seekable().count() ); }
		real64 * real64Data() const { PROC RETURN _data -> real64Data( DalData<T>::seekable().from(), DalData<T>::seekable().count() ); }
		bool8 * boolData() const { PROC RETURN _data -> boolData( DalData<T>::seekable().from(), DalData<T>::seekable().count() ); }
		char* stringData() const { PROC RETURN _data -> stringData(  DalData<T>::seekable().from(),  DalData<T>::seekable().count() ); }

		unsigned int dataSize() const { PROC RETURN _data -> dataType(); }
		//unsigned long elements() const { PROC RETURN _data -> size(); }
		Dal::DataType dataType() const { PROC RETURN _data -> dataType(); }
		void printOn( ostream & os, const std::string & sep="" ) const
		{ PROC _data -> printOn( os, sep ); ENDPROC }


	private :

		DataStorage *			_data;
};


#endif

