using namespace std;
#include<cassert>
#include <Dal.h>
#include <cmath>
#include <Tracer.h>

bool feq(float v1, float v2) { PROC RETURN abs(v1-v2) < 1e-5; }
bool deq(double v1, double v2) { PROC RETURN abs(v1-v2) < 1e-5; }

int main()
{
  unsigned long nRows = 1000;
  unsigned long nElements = 5;
  vector<unsigned long> size;
  size.push_back(nElements);

  DataSet* set = dataSetServer->open("test.dat",DataSet::Read);
  Table* tab = set->table("someTable");
  assert( tab->rows() == nRows );
  
  Column *col;
  ColumnData *d;

  col = tab->column("real32");
  assert( col->size() == size );
  assert( col->elements() == nElements );

  d = col->data();
  assert( d->elements() == nElements );
  assert( d->size() == size );
  assert( d->count() == nRows );

  real32 *r32 = d->real32Data();
  for (unsigned long row = d->from(); row < d->from()+d->count(); ++row)
    for (unsigned long element = 0; element < d->elements(); ++element) {
      unsigned long pos = d->position(row,element);
      assert( pos == row * nElements + element );
      assert( feq( r32[pos], row + 0.1*element ) );
    }
  delete d;

  size.push_back(2);

  col = tab->column("int32");
  assert( col->size() == size );
  assert( col->elements() == size[0]*size[1] );

  d = col->data();
  assert( d->count() == nRows );
  assert( d->elements() == size[0]*size[1]);

  int32* i32 = d->int32Data();

  vector<unsigned long> pos(2);
  for (unsigned long row = d->from(); row < d->from()+d->count(); ++row) {
    for (pos[1] = 0; pos[1] < size[1]; ++pos[1])
      for (pos[0] = 0; pos[0] < size[0]; ++pos[0]) {
        unsigned long idx = d->position(row,pos);
        assert( idx ==  row * 10 + pos[0] * 2 + pos[1] );
        assert( feq( i32[idx], 100*row + 10*pos[0] + pos[1] ) );
      }
  }
  delete d;

  col = tab->column("real64");
  vector<unsigned long> vectorSize;
  vectorSize.push_back( 1000 );
  assert( col->size() == vectorSize );
  assert( col->elements() == 1000 );

  for( unsigned int i = 0; i < tab -> rows(); ++i )
  {
    d = col -> data( i, 1 );
    real64* r64 = d->real64Data();
    for( unsigned long j = 0; j < col -> elements(); ++j )
    {
      assert( deq( r64[ j ], i * col -> elements() + j ));
    } // for
    delete d;
  } // for

  dataSetServer->close(set);

  return 0;
}

