using namespace std;
#include <StopWatch.h>
#include <Dal.h>
#include <iostream>
#include<cassert>

struct Rows
{
	Rows() : _nRows( 0 ), _from( 0 ), _count( 0 ) {}
	unsigned long	_nRows;
	unsigned long	_from;
	unsigned long	_count;

};


template<class T>
static void
insertrows(
	DataSet*		set,
	const Rows&		insertTheseRows,
	T* (Data::*tdata)() const )
{
	Table*			tab = set -> table("some table");
	Column*			col;
	Data*			cd;
	T*			d;
        const unsigned long	nRows = tab -> rows();

	tab -> insertRows( insertTheseRows._from, insertTheseRows._count );
	assert( tab -> rows() == nRows + insertTheseRows._count );

	col = tab -> column( "col" );
	cd = col -> data( insertTheseRows._from, insertTheseRows._count );
	d = (cd ->* tdata)();

	for( unsigned int i = 0; i < insertTheseRows._count; ++i )
	{
		//d[ i + insertTheseRows._from ] = i +  insertTheseRows._from;
		d[ i ] = i +  insertTheseRows._from;
	} // for
	delete cd;
}


template<class T>
static void
fill(
	DataSet*		set,
	unsigned long		nRows,
	Dal::DataType		dataType,
	T* (Data::*tdata)() const )
{

	Table*		tab = set -> addTable( "some table", nRows );
	Column*		col;
	Data*		cd;
	T*		d;

	col = tab -> addColumn( "col", dataType, "data", "dm" );
	cd = col -> data();
	d = (cd ->* tdata)();

	for( unsigned long i = 0; i < nRows; ++i )
	{
		d[ i ] = i;
	} // for
	delete cd;
}


template<class T>
static void
check(
	DataSet*		set,
	const Rows&		insertTheseRows,
	T* (Data::*tdata)() const )
{

	Table*			tab = set -> table( "some table" );
	const unsigned long 	nRows = tab -> rows();
	Column*			col;
	Data*			cd;
	T*			d;

	col = tab -> column( "col" );
	cd = col -> data();
	d = (cd ->* tdata)();

	for( unsigned long i = 0, j  = 0; i < nRows; ++i )
	{
		if( i == insertTheseRows._from && insertTheseRows._count != 0 )
		{
			for( unsigned long k = 0; k < insertTheseRows._count; ++k )
			{
				assert( d[i+k] == T(k+j) );
			} // for
			i += insertTheseRows._count - 1;
		}
		else
		{
			assert( d[i] == T(j) );
			++j;
		} // if
	} // for
	delete cd;
}


template<class T>
static void
test(
	unsigned long		nRows,
	const Rows& 		insertTheseRows,
	Dal::DataType		dataType,
	T* (Data::*tdata)() const )
{
  Rows			none;

  DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
  fill( set, nRows, dataType, tdata );
  check( set, none, tdata );
  insertrows( set, insertTheseRows, tdata );
  check( set, insertTheseRows, tdata );
  dataSetServer -> close( set );

  set = dataSetServer->open("test.dat",DataSet::Read);
  check( set, insertTheseRows, tdata );
  dataSetServer -> close( set );

}


static void
doTest1( unsigned long nRows, const Rows& deleteTheseRows )
{

  test( nRows, deleteTheseRows, Dal::Int8, &Data::int8Data );
  test( nRows, deleteTheseRows, Dal::Uint16, &Data::uint16Data );
  test( nRows, deleteTheseRows, Dal::Int16, &Data::int16Data );
  test( nRows, deleteTheseRows, Dal::Uint32, &Data::uint32Data );
  test( nRows, deleteTheseRows, Dal::Int32, &Data::int32Data );
  test( nRows, deleteTheseRows, Dal::Real32, &Data::real32Data );
  test( nRows, deleteTheseRows, Dal::Real64, &Data::real64Data );
 

}



static void
doTest( unsigned long nRows )
{

  // *********************************
  // Get clock for timing information.
  // *********************************
  StopWatch		cpuWatch( StopWatch::CPU );
  StopWatch		calenderWatch( StopWatch::Calender );

  Rows	rows;

  rows._nRows = nRows;

  rows._from = 0;
  rows._count = 0;
  doTest1( rows._nRows, rows );

  rows._from = 0;
  rows._count = 1;
  doTest1( rows._nRows, rows );

  rows._from = 0;
  rows._count = nRows;
  doTest1( rows._nRows, rows );

  rows._from = nRows - 1;
  rows._count = 0;
  doTest1( rows._nRows, rows );

  rows._from = nRows - 1;
  rows._count = 1;
  doTest1( rows._nRows, rows );

  rows._from = nRows - 1;
  rows._count = nRows;
  doTest1( rows._nRows, rows );

  rows._from = 0;
  rows._count = nRows;
  doTest1( rows._nRows, rows );

  rows._from = nRows;
  rows._count = 0;
  doTest1( rows._nRows, rows );

  rows._from = nRows;
  rows._count = nRows;
  doTest1( rows._nRows, rows );

  if( nRows > 3 )
  {
  	rows._from = nRows - 4;
  	rows._count = 4;
  	doTest1( rows._nRows, rows );

  	rows._from = 0;
  	rows._count = 4;
  } // if

  cout << "CPU time: " << cpuWatch.elapsed() << " secs" << endl;
  cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;
}


int
main()
{
  doTest( 1 );
  doTest( 10 );
  doTest( 100 );
  doTest( 2589 );

}

