program test

  use dal
  use errorhandling
  
  implicit none

  type(DataSetT) set
  type(TableT) tab
  type(ColumnT) col
  logical(kind=bool), dimension(:), pointer :: b
  integer(kind=int8), dimension(:), pointer :: i8
  integer(kind=int16), dimension(:), pointer :: i16
  integer(kind=int32), dimension(:), pointer :: i32
  real(kind=single), dimension(:), pointer :: r32
  real(kind=double), dimension(:), pointer :: r64
  character(len=1024) :: s
  character(len=1024) :: s1

  integer i

  set = dataSet("test.dat",READ)
  if ( .not. hasBlock(set,"some table") ) &
    call error("testFailed","! hasBlock some table")
  if ( hasBlock(set,"noblock") ) &
    call error("testFailed","hasBlock noblock")

  tab = table(set,"some table")
  if (numberOfRows(tab) /= 100) &
    call error("testFailed","wrong nr of rows")
  if ( .not. hasColumn(tab,"bool") ) &
    call error("testFailed","! hasColumn bool")
  if ( hasColumn(tab,"nonexistent") ) &
    call error("testFailed","hasColumn nonexistent") 

  col = column(tab,"bool",READ)
  b => boolData(col)
  if (any(shape(b) /= (/ 100 /))) call error("testFailed","shape b")
  do i=0,numberOfRows(tab)-1
    if (b(i) .neqv. ( modulo (i,2) .eq. 0 ) ) call error("testFailed","value b")
  end do
  call release(col)

  col = column(tab,"int8",READ)
  i8 => int8Data(col)
  if (any(shape(i8) /= (/ 100 /))) call error("testFailed","shape i8")
  do i=0,numberOfRows(tab)-1
    if (i8(i) /= i) call error("testFailed","value i8")
  end do
  call release(col)
    
  col = column(tab,"int16",READ)
  i16 => int16Data(col)
  if (any(shape(i16) /= (/ 100 /))) call error("testFailed","shape i16")
  do i=0,numberOfRows(tab)-1
    if (i16(i) /= 2*i) call error("testFailed","value i16")
  end do
  call release(col)

  col = column(tab,"int32",READ)
  i32 => int32Data(col)
  if (any(shape(i32) /= (/ 100 /))) call error("testFailed","shape i32")
  do i=0,numberOfRows(tab)-1
    if (i32(i) /= 3*i) call error("testFailed","value i32")
  end do
  call release(col)

  col = column(tab,"real32",READ)
  r32 => real32Data(col)
  if (any(shape(r32) /= (/ 100 /))) call error("testFailed","shape r32")
  do i=0,numberOfRows(tab)-1
    if (r32(i) /= 0.5*i) call error("testFailed","value r32")
  end do
  call release(col)

  col = column(tab,"real64",READ)
  r64 => real64Data(col)
  if (any(shape(r64) /= (/ 100 /))) call error("testFailed","shape r64")
  do i=0,numberOfRows(tab)-1
    if (r64(i) /= 0.25*i) call error("testFailed","value r64")
  end do
  call release(col)

  col = column(tab,"string",READ)
  if (any(dimensions(col) /= (/ 80/))) &
	call error("testFailed","shape string column")
  do i=0,numberOfRows(tab)-1
    write(s,*) "string",i
    write(*,*) '"',stringCell(col,i),'"', len(stringCell(col,i))
    write(*,*) '"',trim(stringCell(col,i)),'"', len(trim(stringCell(col,i)))
    write(*,*) '"', s, '"', len(s)
    write(*,*) '"', trim(s), '"', len(trim(s))
    s1 = stringCell(col,i)
    s1 = s1(1:len(trim(s)) )
    write(*,*) s1
    !if (trim(stringCell(col,i)) /= trim(s)) &
    if (trim(s1) /= trim(s) ) &
	call error("testFailed","value string")
  end do

! Test that resetting and reading a string cell works ok.
  call release(set)
  set = dataSet("test.dat",MODIFY)
  tab = table(set,"some table")
  col = column(tab,"string",MODIFY)
  call setStringCell(col,0,s)
  write(*,*) 'Written value: ',trim(s)
  s1 = stringCell(col,0)
  s1 = s1(1:len(trim(s)) )
  write(*,*) 'Read value: ',trim(s1)
  if (trim(s1) /= trim(s) ) &
      call error("testFailed","value string")
  
  call release(set)

end program test
